class GithubBadCredentials(Exception):
    ''' Raised when username/password for github is wrong '''
    def __init__(self, msg):
        self.msg = msg

class GithubRepoNotFound(Exception):
    ''' Raised when the repo is not found for the user '''
    def __init__(self, msg):
        self.msg = msg



import libpagure
from libpagure.libpagure import Pagure
from github import Github

from exceptions import GithubBadCredentials, GithubRepoNotFound


class GithubImporter():
    ''' Imports from Github using PyGithub and libpagure '''


    def __init__(
            self,
            github_username,
            github_password,
            github_project_name,
            pagure_api_key,
            pagure_project_name,
            pagure_username=None,
            instance_url='https://pagure.io'):

        self.github_username = github_username
        self.github_password = github_password
        self.github_project_name = github_project_name
        self.pagure_project_name = pagure_project_name
        self.github = Github(github_username, github_password)
        self.pagure = Pagure(pagure_api_key, pagure_project_name,
                                pagure_username, instance_url)


    def _get_available_issue_id(self):
        ''' Private method which checks the id
        which would be available for the new issue
        '''
        issues = self.pagure.list_issues()
        try:
            return max([int(issue['id']) for issue in issues]) + 1
        except ValueError:
            return 1


    def _get_repo(self, github_user):
        ''' Private method to get the repo object
        using the given github project name
        '''
        repos = github_user.get_repos()
        for repo in repos:
            if repo.name == self.github_project_name:
                return repo
        raise GithubRepoNotFound(
                'No user repository with given github project name found')


    def import_issues(self, status='all'):
        ''' Imports the issues on github for
        the given project
        '''
        github_user = None
        try:
            github_user = self.github.get_user(self.github_username)
        except:
            raise GithubBadCredentials(
                    'Given github credentials are not correct')

        repo = self._get_repo(github_user)
        for github_issue in repo.get_issues(state=status):
            pagure_issue_title = github_issue.title
            if github_issue.body:
                pagure_issue_content = github_issue.body
            else:
                pagure_issue_content = '#No Description Provided'

            issue_id = self._get_available_issue_id()
            self.pagure.create_issue(
                    pagure_issue_title, pagure_issue_content)

            #comments on the issue
            for comment in github_issue.get_comments():
                self.pagure.comment_issue(issue_id, str(comment.body))

            #change status of the issue if closed
            if github_issue.state.lower() == 'closed':
                self.pagure.change_issue_status(issue_id, 'Fixed')




from sources.importer_github import GithubImporter
import getpass

def form_github_issues():
    github_username = raw_input('Enter you Github Username: ')
    github_password = getpass.getpass('Enter your github password: ')
    github_project_name = raw_input('Enter github project name: ')
    pagure_api_key = raw_input('Enter your pagure api key: ')
    pagure_project_name = raw_input('Enter pagure project name: ')

    is_forked = raw_input('Is the pagure project a forked repo ? (y/n): ') or 'n'
    if is_forked.lower() == 'y':
        pagure_username = raw_input('Enter your pagure username: ')
    else:
        pagure_username = None

    is_pagure_io = raw_input(
                'Is the pagure instance url - https://pagure.io ?: (y/n) ') or 'y'
    if is_pagure_io.lower() == 'n':
        pagure_instance = raw_input('Enter the pagure instance url: ') or 'https://pagure.io'
    else:
        pagure_instance = 'https://pagure.io'

    status = raw_input(
            'Enter status of the issues to be imported (all/open/closed): ') or 'all'

    github_importer = GithubImporter(
                        github_username=github_username,
                        github_password=github_password,
                        github_project_name=github_project_name,
                        pagure_api_key=pagure_api_key,
                        pagure_project_name=pagure_project_name,
                        pagure_username=pagure_username,
                        instance_url=pagure_instance)
    github_importer.import_issues(status)



#!/usr/bin/env python

from forms import form_github_issues
from settings import IMPORT_SOURCES, IMPORT_OPTIONS

def github_handler(item):
    if item.lower() == 'issues':
        form_github_issues()
    return

def main():
    source = raw_input('Enter source from where you want to import: ')
    if source.lower() not in IMPORT_SOURCES:
        print 'Source location not supported'
        return

    item = raw_input('Enter the item to be imported: ')
    if item.lower() not in IMPORT_OPTIONS[source]:
        print 'Item import not supported'
        return

    if source.lower() == 'github':
        github_handler(item)

    return

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='pagure_importer',
    packages=['pagure_importer'],
    version='0.0.2',
    description='CLI tool for imports to Pagure',
    author='Vivek Anand',
    author_email='vivekanand1101@gmail.com',
    url='https://pagure.io/pagure-importer',
    keywords=['pagure', 'importer', 'import'],
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',

    ],
    license='GNU General Public License v2.0',
    entry_points={
        'console_scripts': [
            'pgimport = pagure_importer.run:main'
        ],
    },
    include_package_data=True,
    install_requires=read('requirements.txt'),
)
